from .bootstrap import Bootstrap
from .query import QueryBuilder, Query
from .parser import parse
from .logger import get_logger


version = '1.7'


# Expose class methods as the public API


_bootstrap = Bootstrap()
is_bootstrapped = _bootstrap.is_bootstrapped
clear_bootstrapping = _bootstrap.clear_bootstrapping
bootstrap = _bootstrap.bootstrap
save_bootstrap_data = _bootstrap.save_bootstrap_data
load_bootstrap_data = _bootstrap.load_bootstrap_data
bootstrap_is_older_than = _bootstrap.bootstrap_is_older_than


_query_builder = QueryBuilder(_bootstrap)
build_query = _query_builder.build


# Query helpers


def asn(as_number, rir=None, raw=False):
    method, url, exact_match = build_query(
        query_type='asn', query_value=as_number, rir=rir)
    q = Query(method, url)
    response = q.request()
    return response if raw else parse(_bootstrap, 'autnum', response)


def domain(domain_name, raw=False):
    method, url, exact_match = build_query(
        query_type='domain', query_value=domain_name)
    q = Query(method, url)
    response = q.request()
    return response if raw else parse(_bootstrap, 'domain', response)


def ip(ip_address_or_network, rir=None, raw=False):
    method, url, exact_match = build_query(
        query_type='ip', query_value=ip_address_or_network, rir=rir)
    q = Query(method, url)
    response = q.request()
    return response if raw else parse(_bootstrap, 'ip', response)


def entity(entity_handle, rir=None, raw=False):
    method, url, exact_match = build_query(
        query_type='entity', query_value=entity_handle, rir=rir)
    q = Query(method, url)
    response = q.request()
    return response if raw else parse(_bootstrap, 'entity', response)



import requests
from .errors import QueryError, UnsupportedError
from .logger import get_logger


log = get_logger('utils')


user_agent = 'whoisit'


def http_request(url, method='GET', headers={}, data={}, *args, **kwargs):
    '''
        Simple wrapper over requests.
    '''
    methods = ('GET',)
    if method not in methods:
        raise UnsupportedError(f'HTTP methods supported are: {methods}, got: {method}')
    headers['User-Agent'] = user_agent
    log.debug(f'Making HTTP {method} request to {url}')
    try:
        return requests.request(method, url, headers=headers, data=data, *args,
                                **kwargs)
    except Exception as e:
        raise QueryError(f'Failed to make a {method} request to {url}: {e}') from e


def is_subnet_of(network_a, network_b):
    a_len = network_a.prefixlen
    b_len = network_b.prefixlen
    return a_len >= b_len and network_a.supernet(a_len - b_len) == network_b


default_chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-'


def contains_only_chars(s, chars=default_chars):
    for c in s:
        if c not in chars:
            return False
    return True



class WhoisItError(Exception):
    '''
        Parent Exception for all whoisit raised exceptions.
    '''

class BootstrapError(WhoisItError):
    '''
        Raised when there are any issues with bootstrapping.
    '''

class QueryError(WhoisItError):
    '''
        Raised when there are any issues with queries.
    '''

class UnsupportedError(WhoisItError):
    '''
        Raised when a feature in a query is unsupported.
    '''


class ParseError(WhoisItError):
    '''
        Raised when failing to parse response data.
    '''


class ResourceDoesNotExist(WhoisItError):
    '''
        Raised when querying a resource which doesn't exist.
    '''